<?php
/**
* ReadyScript (http://readyscript.ru)
*
* @copyright Copyright (c) ReadyScript lab. (http://readyscript.ru)
* @license http://readyscript.ru/licenseAgreement/
*/
namespace Payanyway\Model\PaymentType;
use \RS\Orm\Type;
use Shop\Model\Orm\Transaction;
use Shop\Model\PaymentType\AbstractType;

/**
* Способ оплаты - PayAnyWay
*/
class Payanyway extends AbstractType
{
    const
        PAYMENT_URL = "https://www.payanyway.ru/assistant.htm",                        // Assistant link
        DEMO_URL    = "https://demo.moneta.ru/assistant.htm";
    
    /**
    * Возвращает название расчетного модуля (типа доставки)
    * 
    * @return string
    */
    function getTitle()
    {
        return t('PayAnyWay');
    }
    
    /**
    * Возвращает описание типа оплаты. Возможен HTML
    * 
    * @return string
    */
    function getDescription()
    {
        return t('Приём платежей через PayAnyWay, v3');
    }
    
    /**
    * Возвращает идентификатор данного типа оплаты. (только англ. буквы)
    * 
    * @return string
    */
    function getShortName()
    {
        return 'payanyway';
    }

    /**
     * Отправка данных с помощью POST?
     *
     */
    function isPostQuery()
    {
        return true;
    }

    /**
    * Возвращает ORM объект для генерации формы или null
    * 
    * @return \RS\Orm\FormObject | null
    */
    function getFormObject()
    {
        $properties = new \RS\Orm\PropertyIterator(array(
            'account_id' => new Type\Varchar(array(
                'maxLength' => 20,
                'description' => t('Account ID - номер расширенного счёта магазина'),

            )),
            'account_code' => new Type\Varchar(array(
                'maxLength' => 255,
                'description' => t('Account code - код проверки целостности данных'),

            )),
            'is_demo' => new Type\Varchar(array(
                'maxLength' => 5,
                'description' => t('Демо режим?'),
                'listFromArray' => array(array(
                    '0' => t('Выключено'),
                    '1' => t('Да, демо режим'),
                ))
            )),
            'is_test' => new Type\Varchar(array(
                'maxLength' => 5,
                'description' => t('Тестовый режим?'),
                'listFromArray' => array(array(
                    '0' => t('Выключено'),
                    '1' => t('Да, тестовый режим'),
                ))
            )),
            'language' => new Type\Varchar(array(
                'maxLength' => 5,
                'description' => t('Язык интерфейса'),
                'listFromArray' => array(array(
                    0    => t('Определяется PayAnyWay'),
                    'ru' => t('Русский'),
                    'ua' => t('Украинский'),
                    'en' => t('Английский'),
                ))
            )),
            '__help__' => new Type\Mixed(array(
                'description' => t(''),
                'visible' => true,
                'template' => '%payanyway%/form/payment/payanyway/help.tpl'
            )),
        ));

        return new \RS\Orm\FormObject($properties);
    }

    /**
     * Возвращает true, если данный тип поддерживает проведение платежа через интернет
     *
     * @return bool
     */
    function canOnlinePay()
    {
        return true;
    }

    /**
    * Возвращает URL для перехода на сайт сервиса оплаты
    * 
    * @param Transaction $transaction
    * @return string
    */
    function getPayUrl(Transaction $transaction)
    {
        $order      = $transaction->getOrder();     // Данные о заказе
        $user       = $order->getUser();            // Пользователь который должен оплатить

        $inv_id     = $transaction->id;
        $out_summ   = number_format($transaction->cost, 2, '.', '');
        $in_cur     = $this->getPaymentCurrency();
        if ($in_cur == 'RUR') {
            $in_cur = 'RUB';
        }

        $params = array();
        $params['MNT_ID']             = $this->getOption('account_id');
        $params['MNT_AMOUNT']         = $out_summ;
        $params['MNT_CURRENCY_CODE']  = $in_cur;
        $params['MNT_TRANSACTION_ID'] = $inv_id;
        $params['MNT_SUBSCRIBER_ID']  = $user['e_mail'];
        $params['MNT_DESCRIPTION']    = t("Оплата заказа №").$order['order_num'];

        $is_test = $this->getOption('is_test');
        if (!$is_test) {
            $is_test = '0';
        }

        // подпись запроса
        // MNT_SIGNATURE = MD5(MNT_ID + MNT_TRANSACTION_ID + MNT_AMOUNT + MNT_CURRENCY_CODE + MNT_SUBSCRIBER_ID + ТЕСТОВЫЙ РЕЖИМ + КОД ПРОВЕРКИ ЦЕЛОСТНОСТИ ДАННЫХ)
        $params['MNT_SIGNATURE']      = md5( $params['MNT_ID'] . $params['MNT_TRANSACTION_ID'] . $params['MNT_AMOUNT'] . $params['MNT_CURRENCY_CODE'] . $params['MNT_SUBSCRIBER_ID'] . $is_test . $this->getOption('account_code') );
        $this->addPostParams($params); // Добавляем параметры для POST запроса
        if (intval($this->getOption('is_demo')) == 1) {
            $url = self::DEMO_URL;
        }
        else {
            $url = self::PAYMENT_URL;
        }

        return $url;    // url пост запроса
    }
    
    /**
    * Возвращает дополнительные параметры для печати чека по ФЗ-54
    * 
    * @param \Shop\Model\Orm\Transaction $transaction
    * @return array|false
    */
    protected function getParamsForFZ54Check($MNT_TRANSACTION_ID)
    {
        $rub_currency = \Catalog\Model\CurrencyApi::getByUid('RUB');
        if (! $rub_currency ) {
            return false;
        }

        // поднять данные с БД
        $transaction = new \Shop\Model\Orm\Transaction($MNT_TRANSACTION_ID);

        // данные для кассы
        $kassa_inventory = null;
        $kassa_delivery = 0;
        $inventory = array();
        if ($transaction && isset($transaction['order_id'])) {
            $order = $transaction->getOrder();
            $cart = $order->getCart();
            if ($cart) {
                $address = $order->getAddress();
                $tax_api = new \Shop\Model\TaxApi();
                $products = $cart->getProductItems();
                foreach ($products as $product) {
                    $taxes = $tax_api->getProductTaxes($product['product'], $this->transaction->getUser(), $address);
                    $item = array(
                        'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($product['product']['title']))),
                        'price' => round(($product['cartitem']['price'] - $product['cartitem']['discount']) * $rub_currency['ratio'], 2),
                        'quantity' => $product['cartitem']['amount'],
                        'vatTag' => $this->getRightTax($taxes, $address),
                    );
                    $inventory[] = $item;
                }
                $delivery = $cart->getCartItemsByType(\Shop\Model\Cart::TYPE_DELIVERY);
                foreach ($delivery as $delivery_item) {
                    $kassa_delivery = $kassa_delivery + round(($delivery_item['price'] - $delivery_item['discount']) * $rub_currency['ratio'], 2);
                }
            }

            $transaction['receipt'] = $transaction::RECEIPT_SUCCESS;
            $transaction->update();

            if ($order) {
                $order['status'] = $transaction->getPayment()->success_status;
                $order->update();
            }

        } else {
            // Пополнение лицевого счета
            $item = array(
                'name' => $transaction->reason,
                'price' => round($transaction->cost * $rub_currency['ratio'], 2),
                'quantity' => 1,
                'vatTag' => $this->getOption('nds_personal_account')
            );
            $inventory[] = $item;
        }
        if (count($inventory)) {
            $kassa_inventory = json_encode($inventory);
            return array("kassa_inventory" => $kassa_inventory, "kassa_delivery" => $kassa_delivery);
        }
        else {
            return false;
        }

    }

    function itemName($product)
    {
        if ($product['product']['barcode']){
            $result = $product['product']['barcode'];
        }
        $result = $result.' '.$product['product']['title'];
        if (iconv_strlen($result)>64){
            $result = iconv_substr($result, 0 , 61 , 'UTF-8' );
            $result = $result.'...';
        }

        return $result;
    }

    /**
     * Получает трех символьный код базовой валюты в которой ведётся оплата
     *
     */
    private function getPaymentCurrency()
    {
        /**
         * @var \Catalog\Model\Orm\Currency
         */
        $currency = \RS\Orm\Request::make()
            ->from(new \Catalog\Model\Orm\Currency())
            ->where(array(
                'public'  => 1,
                'is_base'  => 1,
            ))
            ->object();
        return $currency ? $currency->title : false;
    }

    /**
    * Возвращает ID заказа исходя из REQUEST-параметров соотвествующего типа оплаты
    * Используется только для Online-платежей
    * 
    * @return mixed
    */
    function getTransactionIdFromRequest(\RS\Http\Request $request)
    {
        return $request->request('MNT_TRANSACTION_ID', TYPE_STRING);
    }

    function onResult(\Shop\Model\Orm\Transaction $transaction, \RS\Http\Request $request)
    {
        // Получаем переменные запроса
        $MNT_ID             = $request->request('MNT_ID', TYPE_STRING);
        $MNT_TRANSACTION_ID = $request->request('MNT_TRANSACTION_ID', TYPE_STRING);
        $MNT_OPERATION_ID   = $request->request('MNT_OPERATION_ID', TYPE_STRING);
        $MNT_AMOUNT         = $request->request('MNT_AMOUNT', TYPE_STRING);
        $MNT_CURRENCY_CODE  = $request->request('MNT_CURRENCY_CODE', TYPE_STRING);
        $MNT_SUBSCRIBER_ID  = $request->request('MNT_SUBSCRIBER_ID', TYPE_STRING);
        $MNT_TEST_MODE      = $request->request('MNT_TEST_MODE', TYPE_STRING);
        $MNT_SIGNATURE      = $request->request('MNT_SIGNATURE', TYPE_STRING);

        if (!$MNT_TEST_MODE) {
            $MNT_TEST_MODE = '0';
        }

        // подпись запроса
        // MNT_SIGNATURE = MD5(MNT_ID + MNT_TRANSACTION_ID + MNT_OPERATION_ID + MNT_AMOUNT + MNT_CURRENCY_CODE + MNT_SUBSCRIBER_ID + MNT_TEST_MODE + КОД ПРОВЕРКИ ЦЕЛОСТНОСТИ ДАННЫХ)
        $signature = md5( $MNT_ID .  $MNT_TRANSACTION_ID . $MNT_OPERATION_ID . $MNT_AMOUNT . $MNT_CURRENCY_CODE . $MNT_SUBSCRIBER_ID . $MNT_TEST_MODE . $this->getOption('account_code') );

        if ($signature != $MNT_SIGNATURE) {
            $exception = new \Shop\Model\PaymentType\ResultException(t('FAIL'));
            $exception->setResponse('FAIL');
            $exception->setUpdateTransaction(false);
            throw $exception;
        }

        // здесь надо поднимать номенклатуру и формировать xml ответ
        $kassa_data = $this->getParamsForFZ54Check($MNT_TRANSACTION_ID);
        if ($kassa_data) {

            // сформируем xml ответ
            $resultCode = 200;

            $kassa_inventory = $kassa_data['kassa_inventory'];
            $kassa_customer = $MNT_SUBSCRIBER_ID;
            $kassa_delivery = $kassa_data['kassa_delivery'];

            header("Content-type: application/xml");
            $signature = md5($resultCode . $MNT_ID . $MNT_TRANSACTION_ID . $this->getOption('account_code'));
            $result = '<?xml version="1.0" encoding="UTF-8" ?>';
            $result .= '<MNT_RESPONSE>';
            $result .= '<MNT_ID>' . $MNT_ID . '</MNT_ID>';
            $result .= '<MNT_TRANSACTION_ID>' . $MNT_TRANSACTION_ID . '</MNT_TRANSACTION_ID>';
            $result .= '<MNT_RESULT_CODE>' . $resultCode . '</MNT_RESULT_CODE>';
            $result .= '<MNT_SIGNATURE>' . $signature . '</MNT_SIGNATURE>';

            if ($kassa_inventory || $kassa_customer || $kassa_delivery) {
                $result .= '<MNT_ATTRIBUTES>';
            }

            if ($kassa_inventory) {
                $result .= '<ATTRIBUTE>';
                $result .= '<KEY>INVENTORY</KEY>';
                $result .= '<VALUE>' . $kassa_inventory . '</VALUE>';
                $result .= '</ATTRIBUTE>';
            }

            if ($kassa_customer) {
                $result .= '<ATTRIBUTE>';
                $result .= '<KEY>CUSTOMER</KEY>';
                $result .= '<VALUE>' . $kassa_customer . '</VALUE>';
                $result .= '</ATTRIBUTE>';
            }

            if ($kassa_delivery) {
                $result .= '<ATTRIBUTE>';
                $result .= '<KEY>DELIVERY</KEY>';
                $result .= '<VALUE>' . $kassa_delivery . '</VALUE>';
                $result .= '</ATTRIBUTE>';
            }

            if ($kassa_inventory || $kassa_customer || $kassa_delivery) {
                $result .= '</MNT_ATTRIBUTES>';
            }

            $result .= '</MNT_RESPONSE>';

            echo $result;

        }
        else {
            return 'SUCCESS';
        }

    }

    /**
     * Вызывается при открытии страницы неуспешного проведения платежа
     * Используется только для Online-платежей
     *
     * @param \Shop\Model\Orm\Transaction $transaction
     * @param \RS\Http\Request $request
     * @return void
     */
    function onFail(\Shop\Model\Orm\Transaction $transaction, \RS\Http\Request $request)
    {
        $transaction['status'] = $transaction::STATUS_FAIL;
        $transaction->update();
    }

    /**
    * Возвращает правильный идентификатор налога
    * 
    * @param array(\Shop\Model\Orm\Tax) $taxes - массив налогов
    * @param \Shop\Model\Orm\Address $address - адрес используемый для расчёта
    */
    private function getRightTax(array $taxes, \Shop\Model\Orm\Address $address)
    {
        $tax = new \Shop\Model\Orm\Tax();
        foreach ($taxes as $item){
            if ($item['is_nds']){
                $tax = $item;
                break; 
            } 
        }
        
        //Получим ставку
        $tax_rate = $tax->getRate($address);
        switch((int) $tax_rate){
            case 10:
                $tax_id = ($tax['included']) ? 1107 : 1103;
                break;
            case 18:
                $tax_id = ($tax['included']) ? 1106 : 1102;
                break;
            case 0:
                $tax_id = 1104;
                break;
            default:
                $tax_id = 1105;
        }
        return $tax_id;
    }
    
}
